#!/usr/bin/env python

# Copyright 2016 NeuroData (http://neurodata.io)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# graph.py
# Created by Greg Kiar on 2016-01-27.
# Email: gkiar@jhu.edu

from __future__ import print_function

from itertools import combinations
from collections import defaultdict
import numpy as np
import networkx as nx
import nibabel as nb
import ndmg
import time


class graph(object):
    def __init__(self, N, rois, attr=None):
        """
        Initializes the graph with nodes corresponding to the number of ROIs

        **Positional Arguments:**

                N:
                    - Number of rois
                rois:
                    - Set of ROIs as either an array or niftii file)
                attr:
                    - Node or graph attributes. Can be a list. If 1 dimensional
                      will be interpretted as a graph attribute. If N
                      dimensional will be interpretted as node attributes. If
                      it is any other dimensional, it will be ignored.
        """
        self.N = N
        self.edge_dict = defaultdict(int)

        self.rois = nb.load(rois).get_data()
        n_ids = np.unique(self.rois)
        n_ids = n_ids[n_ids != 0]

        self.g = nx.Graph(name="Generated by NeuroData's MRI Graphs (ndmg)",
                          version=ndmg.version,
                          date=time.asctime(time.localtime()),
                          source="http://m2g.io",
                          region="brain",
                          sensor="Diffusion MRI",
                          ecount=0,
                          vcount=len(n_ids)
                          )
        print(self.g.graph)

        [self.g.add_node(ids) for ids in n_ids]
        pass

    def make_graph(self, streamlines, attr=None):
        """
        Takes streamlines and produces a graph

        **Positional Arguments:**

                streamlines:
                    - Fiber streamlines either file or array in a dipy EuDX
                      or compatible format.
        """
        nlines = np.shape(streamlines)[0]
        print("# of Streamlines: " + str(nlines))

        for idx, streamline in enumerate(streamlines):
            if (idx % int(nlines*0.05)) == 0:
                print(idx)

            points = np.round(streamline).astype(int)
            p = set()
            for point in points:
                try:
                    loc = self.rois[point[0], point[1], point[2]]
                except IndexError:
                    pass
                else:
                    pass

                if loc:
                    p.add(loc)

            edges = combinations(p, 2)
            for edge in edges:
                lst = tuple([int(node) for node in edge])
                self.edge_dict[tuple(sorted(lst))] += 1

        edge_list = [(k[0], k[1], v) for k, v in self.edge_dict.items()]
        self.g.add_weighted_edges_from(edge_list)
        pass

    def get_graph(self):
        """
        Returns the graph object created
        """
        try:
            return self.g
        except AttributeError:
            print("Error: the graph has not yet been defined.")
            pass

    def save_graph(self, graphname, fmt='gpickle'):
        """
        Saves the graph to disk

        **Positional Arguments:**

                graphname:
                    - Filename for the graph

        **Optional Arguments:**

                fmt:
                    - Output graph format
        """
        self.g.graph['ecount'] = nx.number_of_edges(self.g)
        if fmt == 'gpickle':
            nx.write_gpickle(self.g, graphname)
        elif fmt == 'graphml':
            nx.write_graphml(self.g, graphname)
        else:
            raise ValueError('graphml is the only format currently supported')
        pass

    def summary(self):
        """
        User friendly wrapping and display of graph properties
        """
        print("\n Graph Summary:")
        print(nx.info(self.g))
        pass



#!/usr/bin/env python

# Copyright 2014 Open Connectome Project (http://openconnecto.me)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# graph_qc.py
# Created by Greg Kiar on 2016-05-11.
# Email: gkiar@jhu.edu

from __future__ import print_function

from collections import OrderedDict

import networkx as nx
import os


def loadGraphs(filenames, verb=False):
    """
    Given a list of files, returns a dictionary of graphs

    Required parameters:
        filenames:
            - List of filenames for graphs
    Optional parameters:
        verb:
            - Toggles verbose output statements
    """
    #  Initializes empty dictionary
    if type(filenames) is not list:
        filenames = [filenames]
    gstruct = OrderedDict()
    for idx, files in enumerate(filenames):
        if verb:
            print("Loading: " + files)
        #  Adds graphs to dictionary with key being filename
        fname = os.path.basename(files)
        try:
            gstruct[fname] = nx.read_graphml(files)
        except:
            gstruct[fname] = nx.read_gpickle(files)
    return gstruct



#!/usr/bin/env python

from argparse import ArgumentParser
from plotly.offline import download_plotlyjs, init_notebook_mode, iplot, plot
import pickle
import plotly_panels as pp
import os


def make_panel_plot(basepath, outf, dataset=None, atlas=None, scree=True,
                    log=True):
    fnames = [name for name in os.listdir(basepath)
              if os.path.splitext(name)[1] == '.pkl']
    fnames = sorted(fnames)
    paths = [os.path.join(basepath, item) for item in fnames]
    keys = ["_".join(n.split('.')[0].split('_')[1:]) for n in fnames]
    relprob = 'Relative Probability'
    ylabs = [relprob, relprob, relprob, relprob,
             'Eigenvalue', 'Count', relprob, 'Portion of Total Variance']
    xlabs = ['Betweenness Centrality', 'Clustering Coefficient', 'Degree',
             'Edge Weight', 'Dimension', 'Number of Non-zeros',
             'Locality Statistic-1', 'Dimension']

    traces = list(())
    for idx, curr in enumerate(paths):
        f = open(curr)
        dat = pickle.load(f)[keys[idx]]
        f.close()
        if keys[idx] == 'number_non_zeros':
            fig = pp.plot_rugdensity(dat.values())
        elif keys[idx] == 'eigen_sequence' or keys[idx] == 'scree_eigen':
            dims = len(dat.values()[0])
            fig = pp.plot_series(dat.values())
        else:
            xs = dat['xs'].values()
            ys = dat['pdfs'].values()
            fig = pp.plot_density(xs, ys)
        traces += [pp.fig_to_trace(fig)]

    multi = pp.traces_to_panels(traces)
    for idx, curr, in enumerate(paths):
        key = 'axis%d' % (idx+1)
        d = multi.layout['x'+key]['domain']
        multi.layout['x'+key]['domain'] = [d[0], d[1]-0.0125]
        multi.layout['y'+key]['zeroline'] = False
        multi.layout['x'+key]['zeroline'] = False
        multi.layout['x'+key]['title'] = xlabs[idx]
        multi.layout['y'+key]['title'] = ylabs[idx]
        multi.layout['x'+key]['nticks'] = 3
        multi.layout['y'+key]['nticks'] = 3
        if idx in [0, 2, 3, 6]:
            if log:
                multi.layout['x'+key]['type'] = 'log'
                multi.layout['x'+key]['title'] += ' (log scale)'
        if idx in [4, 7]:
            multi.layout['x'+key]['range'] = [1, dims]
            multi.layout['x'+key]['tickvals'] = [1, dims/2, dims]
            if idx in [7]:
                multi.layout['y'+key]['range'] = [0, 1]
                multi.layout['y'+key]['tickvals'] = [0, 0.5, 1]
        if idx in [1]:
            multi.layout['x'+key]['range'] = [0, 1]

    if dataset is not None and atlas is not None:
        if atlas == 'desikan':
            atlas = atlas.capitalize()
        tit = dataset + ' Dataset (' + atlas + ' parcellation)'
    else:
        tit = None
    multi.layout['title'] = tit
    # iplot(multi, validate=False)

    if not scree:
        multi.data[-1] = {}
        multi.layout['x'+key]['title'] = ''
        multi.layout['y'+key]['title'] = ''
        multi = pp.panel_invisible(multi, 8)
        
    plot(multi, validate=False, filename=outf+'.html')


def main():
    parser = ArgumentParser(description="This is a graph qc plotting tool.")
    parser.add_argument("basepath", action="store", help="qc metric .pkl dir")
    parser.add_argument("dataset", action="store", help="dataset name")
    parser.add_argument("atlas", action="store", help="atlas name")
    parser.add_argument("outf", action="store", help="outfile name for plot")
    r = parser.parse_args()

    make_panel_plot(r.basepath, r.outf, r.dataset, r.atlas)

if __name__ == "__main__":
    main()



# Copyright 2014 Open Connectome Project (http://openconnecto.me)
#
# Licensed under the Apache License,  Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing,  software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,  either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ndio.remote.ndingest as NI


def main():

    ni = NI.NDIngest()

    """
    Dataset
    """
    dataset_name = 'MNI152'
    imagesize = (182, 218, 182)
    voxelres = (1000000.0, 1000000.0, 1000000.0)
    offset = (0, 0, 0)
    timerange = (0, 199)
    scalinglevels = 0
    scaling = 1
    public = 1
    metadata = ""

    # Add data set information to ingest object
    ni.add_dataset(dataset_name, imagesize, voxelres, offset,
                   timerange, scalinglevels, scaling)
    ni.add_metadata(metadata)

    """
    Project
    """
    project_name = 'KKI2009dti'
    token_name = 'KKI2009dti'

    # Adds project information to ingest object
    ni.add_project(project_name, token_name, public)

    """
    Channel
    """
    # Sets up general channel info
    datatype = 'uint8'
    channel_type = 'timeseries'
    exceptions = 0
    resolution = 0
    windowrange = (0, 0)
    readonly = 0
    data_url = 'http://openconnecto.me/mrdata/share/ingest'
    file_format = 'SLICE'
    file_type = 'png'

    # Lists channel names
    channels = ['113_1', '113_2', '127_1', '127_2', '142_1', '142_2', '239_1',
                '239_2', '346_1', '346_2', '422_1', '422_2', '492_1', '492_2',
                '502_1', '502_2', '505_1', '505_2', '656_1', '656_2', '679_1',
                '679_2', '742_1', '742_2', '800_1', '800_2', '814_1', '814_2',
                '815_1', '815_2', '849_1', '849_2', '906_1', '906_2', '913_1',
                '913_2', '916_1', '916_2', '934_1', '934_2', '959_1', '959_2']

    # Adds each channel's information to ingest object
    for channel_name in channels:
        ni.add_channel(channel_name, datatype, channel_type, data_url,
                       file_format, file_type, exceptions, resolution,
                       windowrange, readonly)

    # Post the data
    ni.post_data(verifytype='Folder')


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Copyright 2016 NeuroData (http://neurodata.io)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# m2g_startup.py
# Created by Greg Kiar on 2016-01-12.
# Email: gkiar@jhu.edu
# Copyright (c) 2016. All rights reserved

from argparse import ArgumentParser
import os
import sys
import glob


def setup(inDir, dtiListFile, bvalListFile, bvecListFile, mprageListFile):
    # Create lists of files
    dti_types = ('*DTI.nii', '*DTI.nii.gz')

    dtiFiles = get_files(dti_types, inDir)

    bval_types = ('*.b', '*.bval')
    bvalFiles = get_files(bval_types, inDir)

    bvec_types = ('*.bvec', '*.grad')
    bvecFiles = get_files(bvec_types, inDir)

    mprage_types = ('*MPRAGE.nii', '*MPRAGE.nii.gz')
    mprageFiles = get_files(mprage_types, inDir)

    # Writes lists to disk
    write_files(dtiListFile, dtiFiles)
    write_files(bvalListFile, bvalFiles)
    write_files(bvecListFile, bvecFiles)
    write_files(mprageListFile, mprageFiles)


def get_files(ftypes, inDir):
    return [y for x in os.walk(inDir) for z in ftypes
            for y in glob.glob(os.path.join(x[0], z))]


def write_files(outfile, filelist):
    with open(outfile, 'wb') as thefile:
        for item in filelist:
            thefile.write("%s\n" % item)


def main():
    parser = ArgumentParser(description="")
    parser.add_argument("inDir", action="store",
                        help="Input directory for raw data")
    parser.add_argument("dtiListFile", action="store", help="")
    parser.add_argument("bvalListFile", action="store", help="")
    parser.add_argument("bvecListFile", action="store", help="")
    parser.add_argument("mprageListFile", action="store", help="")
    result = parser.parse_args()

    setup(result.inDir, result.dtiListFile, result.bvalListFile,
          result.bvecListFile, result.mprageListFile)

if __name__ == "__main__":
    main()



from distutils.core import setup
from setuptools import setup
import ndmg

VERSION = ndmg.version

setup(
    name='ndmg',
    packages=[
        'ndmg',
        'ndmg.preproc',
        'ndmg.register',
        'ndmg.track',
        'ndmg.graph',
        'ndmg.stats',
        'ndmg.utils',
        'ndmg.scripts'
    ],
    scripts = [
        'ndmg/scripts/ndmg_demo-dwi',
        'ndmg/scripts/ndmg_demo-qc'
    ],
    entry_points = {
        'console_scripts': [
            'ndmg_pipeline=ndmg.scripts.ndmg_pipeline:main',
            'ndmg_bids=ndmg.scripts.ndmg_bids:main'
    ]
    },
    version=VERSION,
    description='Neuro Data MRI to Graphs Pipeline',
    author='Greg Kiar and Will Gray Roncal',
    author_email='gkiar@jhu.edu, wgr@jhu.edu',
    url='https://github.com/neurodata/ndmg',
    download_url='https://github.com/neurodata/ndmg/tarball/' + VERSION,
    keywords=[
        'connectome',
        'mri',
        'pipeline'
    ],
    classifiers=[],
    install_requires=[  # We didnt put versions for numpy, scipy, b/c travis-ci
        'networkx>=1.11',
        'nibabel>=2.0',
        'nilearn>=0.2',
        'sklearn>=0.0',
        'numpy',  # We use nump v1.10.4
        'scipy',  # We use 0.17.0
        'dipy>=0.1',
        'boto3',
        'plotly',
    ]
)



#!/usr/bin/env python

# Copyright 2014 Open Connectome Project (http://openconnecto.me)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

# nifti_to_png.py
# Created by Greg Kiar on 2016-06-13.
# Email: gkiar@jhu.edu

from argparse import ArgumentParser
from scipy.misc import imsave
import nibabel as nb
import os


def convert(indir, outdir,  verbose=False):
    """
    Takes in nifti images, creates directory structure desired by ndstore
    ingest, and then converts niftis to png stacks. This script should be
    called prior to ingesting MR data into ndstore.
    """

    # Create output directory structure
    if verbose:
        print "Creating", outdir, "..."
    os.system('mkdir -p '+outdir)

    for path, dirs, files in os.walk(indir):
        for idx, fl in enumerate(files):
            if verbose:
                print "Loading subject to learn number of time points..."
            im = nb.load(path+fl)
            dat = im.get_data()
            ntime = dat.shape[3]

            base = os.path.splitext(os.path.splitext(fl)[0])[0]
            chan = "_".join(base.split('_')[1:3])
            if verbose:
                print "File:", fl
                print "Channel:", chan
                print "Time steps:", ntime
                print "Creating", outdir + "/" + chan, "..."
            os.system('mkdir -p ' + outdir + "/" + chan)

            for count in range(int(ntime)):
                dirname = outdir + "/" + chan + "/time%04d" % count
                if verbose:
                    print "Creating", dirname, "..."
                os.system('mkdir -p ' + dirname)
                for slices in range(dat.shape[2]):
                    if verbose:
                        print "Saving slice:", slices
                    imsave(dirname + '/%04d.png' % slices,
                           dat[:, :, slices, count].astype('float32').T)


def main():
    parser = ArgumentParser(description="")
    parser.add_argument("indir", action="store", help="directory for niftis")
    parser.add_argument("outdir",  action="store", help="directory for pngs")
    parser.add_argument("-v", "--verbose", action="store_true", default=False,
                        help="Toggles output text")
    result = parser.parse_args()

    convert(result.indir, result.outdir, result.verbose)


if __name__ == "__main__":
    main()
